﻿/*
	简易昼夜系统
	功能：
	1，简易昼夜交替
	
	todo：
	1，实现简易天气效果
	（闪电、刮风、下雨、打雷、子夜等效果）
	
    bug：1，仅需改变G_Time即可改变全局动画播放速度，但因为主光旋转没有精度同步，所以主光的体积光没有跟上环境色改变。

	技术支持：
	AS(2.7.3)、图形补丁(6.5.9)_滴风

    更新：
    修改限制在单人模式下才能使用，局域网可以在进场景后输入隐藏命令：AI 即可
    新增场景光影是否开启判断

	过山河。
    v1.0.1
    20250531

*/
#include "../PScriptLib/AsHelper.as"
#include "../PScriptLib/ValueAnimateManager.as"

// 总动画时长,取时间尽量是2的倍数
const float G_Time = 3600.0f; 

//单人模式检测
bool g_said = false;
// 循环起始时间
float g_cycleStartTime = 0.0f;

void SetupDayNightAnimate() {

// 主光亮度、颜色、阴影强度分段动画
const float HALF_Time = G_Time / 2;
    
// 定义各阶段时间点占G_Time的百分比
const float PHASE1 = G_Time * 0.08333; 
const float PHASE2 = G_Time * 0.16666;
const float PHASE3 = G_Time * 0.50000;
const float PHASE4 = G_Time * 0.16666;
const float PHASE5 = G_Time * 0.08333;

// 计算各阶段时间点
const float PHASE1_START = 0.0f;
const float PHASE1_END = PHASE1_START + PHASE1;
const float PHASE2_START = PHASE1_END;
const float PHASE2_END = PHASE2_START + PHASE2;
const float PHASE3_START = PHASE2_END;
const float PHASE3_END = PHASE3_START + PHASE3;
const float PHASE4_START = PHASE3_END;
const float PHASE4_END = PHASE4_START + PHASE4;
const float PHASE5_START = PHASE4_END;
const float PHASE5_END = PHASE5_START + PHASE5;

    
    Ddfix::SetRenderEnabled(true);
    // 强制开启渲染器
    Ddfix::IsRenderEnabled();
    if (Ddfix::IsRenderEnabled() == false) {
        Ddfix::SetRenderEnabled(true);
    }
    // 隐藏游戏UI
    Ddfix::SetArenaUIVisible(false); 

    //================ 环境光动画 ================
    // 阶段1：快速变亮，占总时长的8.33333%
    ValueAnimateBezierVector3@ ambientPhase1 = ValueAnimateBezierVector3();
    ambientPhase1.duration = PHASE1_END - PHASE1_START;
    ambientPhase1.delay = PHASE1_START;
    //夜间初始颜色RGB
    ambientPhase1.beginValue = Vector3(35,35,60);
    ambientPhase1.endValue = Vector3(241,241,241);
    ambientPhase1.controlPoint = ambientPhase1.beginValue; 
    @ambientPhase1.callback = function(Vector3 v) { 
        Ddfix::SetAmbientColor(v); 
    };

    // 阶段2：保持亮度
    ValueAnimateBezierVector3@ ambientPhase2 = ValueAnimateBezierVector3();
    ambientPhase2.duration = PHASE2_END - PHASE2_START;
    ambientPhase2.delay = PHASE2_START;
    ambientPhase2.beginValue = ambientPhase1.endValue; 
    ambientPhase2.endValue = Vector3(245,245,245);
    ambientPhase2.controlPoint = ambientPhase2.beginValue; 
    @ambientPhase2.callback = function(Vector3 v) { 
        Ddfix::SetAmbientColor(v); 
    };

    // 阶段3：缓慢变暗
    ValueAnimateBezierVector3@ ambientPhase3 = ValueAnimateBezierVector3();
    ambientPhase3.duration = PHASE3_END - PHASE3_START;
    ambientPhase3.delay = PHASE3_START;
    ambientPhase3.beginValue = ambientPhase2.endValue;
    ambientPhase3.endValue = Vector3(210,210,225);
    ambientPhase3.controlPoint = ambientPhase3.beginValue;
    @ambientPhase3.callback = function(Vector3 v) { 
        Ddfix::SetAmbientColor(v); 
    };

    // 阶段4：黄昏
    ValueAnimateBezierVector3@ ambientPhase4 = ValueAnimateBezierVector3();
    ambientPhase4.duration = PHASE4_END - PHASE4_START;
    ambientPhase4.delay = PHASE4_START;
    ambientPhase4.beginValue = ambientPhase3.endValue;
    ambientPhase4.endValue = Vector3(255,160,80); 
    ambientPhase4.controlPoint = ambientPhase4.beginValue;
    @ambientPhase4.callback = function(Vector3 v) { 
        Ddfix::SetAmbientColor(v); 
    };

    // 阶段5：快速变暗
    ValueAnimateBezierVector3@ ambientPhase5 = ValueAnimateBezierVector3();
    ambientPhase5.duration = PHASE5_END - PHASE5_START;
    ambientPhase5.delay = PHASE5_START;
    ambientPhase5.beginValue = ambientPhase4.endValue; 
    ambientPhase5.endValue = Vector3(35,35,60);
    ambientPhase5.controlPoint = ambientPhase5.beginValue; 
    @ambientPhase5.callback = function(Vector3 v) { 
        Ddfix::SetAmbientColor(v); 
    };

    //================ 主光亮度 ================
    ValueAnimateFloat@ brightnessPhase1 = ValueAnimateFloat();
    brightnessPhase1.duration = HALF_Time;
    brightnessPhase1.beginValue = 0.7;
    brightnessPhase1.endValue = 0.8;
    @brightnessPhase1.callback = function(float v){
        Ddfix::SelectLight("pingXingGuang");
        Ddfix::SetLightBrightness(v);
    };

    ValueAnimateFloat@ brightnessPhase2 = ValueAnimateFloat();
    brightnessPhase2.duration = HALF_Time;
    brightnessPhase2.delay = HALF_Time;
    brightnessPhase2.beginValue = 0.9;
    brightnessPhase2.endValue = 1.0;
    @brightnessPhase2.callback = brightnessPhase1.callback;

    //================ 主光颜色 ================
    ValueAnimateVector3@ colorPhase1 = ValueAnimateVector3();
    colorPhase1.duration = HALF_Time;
    colorPhase1.beginValue = Vector3(25,25,0);
    colorPhase1.endValue = Vector3(248,248,255);
    @colorPhase1.callback = function(Vector3 v){
        Ddfix::SelectLight("pingXingGuang");
        Ddfix::SetLightColor(v);
    };

    ValueAnimateVector3@ colorPhase2 = ValueAnimateVector3();
    colorPhase2.duration = HALF_Time;
    colorPhase2.delay = HALF_Time;
    colorPhase2.beginValue = Vector3(248,248,255);
    colorPhase2.endValue = Vector3(25,25,0);
    @colorPhase2.callback = colorPhase1.callback;

    //================ 主光模拟太阳光位移 ================
    ValueAnimateBezierVector3@ moveAnim = ValueAnimateBezierVector3();
    moveAnim.duration = G_Time;
    // 日出点(xyz)
    moveAnim.beginValue = Vector3(10000, 0, 50);  
    // 弧顶控制点  
    moveAnim.controlPoint = Vector3(0, 0, 10000);
    // 日落点       
    moveAnim.endValue = Vector3(-8000, 0, 50);        
    @moveAnim.callback = function(Vector3 pos){
        Ddfix::SelectLight("pingXingGuang");
        Ddfix::SetLightPosition(pos);
    };

    //================ 主光旋转动画 ================
    ValueAnimateFloat@ rotateAnim = ValueAnimateFloat();
    rotateAnim.duration = G_Time;
    rotateAnim.beginValue = -1.0;
    rotateAnim.endValue = 1.001;
    @rotateAnim.callback = function(float x){
        float z = x * x - 1.0;
        Ddfix::SelectLight("pingXingGuang");
        //如果想要修改旋转方向，直接改y轴即可，比如(x, 0.5,z)，进游戏观察
        Ddfix::SetLightRotationArc(Vector3(x, 0.0, z));
    };

    //================ 雾动画 ================
// 雾颜色
Ddfix::SetFogColor(Vector3(180,190,200));

ValueAnimateVector2@ fogPhase1 = ValueAnimateVector2();
fogPhase1.duration = HALF_Time;
fogPhase1.beginValue = Vector2(0, 0.2);
fogPhase1.endValue = Vector2(0, 0.45); 
@fogPhase1.callback = function(Vector2 value){
Ddfix::SetFogDistance(value.x, value.y);
};

ValueAnimateVector2@ fogPhase2 = ValueAnimateVector2();
fogPhase2.duration = HALF_Time;
fogPhase2.delay = HALF_Time; 
fogPhase2.beginValue = Vector2(0, 0.45); 
fogPhase2.endValue = Vector2(0, 0.2);
@fogPhase2.callback = fogPhase1.callback; 

    //================ 提交所有动画 ================
    //环境光
    g_valueAnimateManager.Add(ambientPhase1);
    g_valueAnimateManager.Add(ambientPhase2);
    g_valueAnimateManager.Add(ambientPhase3);
    g_valueAnimateManager.Add(ambientPhase4);
    g_valueAnimateManager.Add(ambientPhase5);
    //主光亮度
    g_valueAnimateManager.Add(brightnessPhase1);
    g_valueAnimateManager.Add(brightnessPhase2);
    //颜色
    g_valueAnimateManager.Add(colorPhase1);
    g_valueAnimateManager.Add(colorPhase2);
    //主光体积光坐标
    g_valueAnimateManager.Add(moveAnim);
    //旋转
    g_valueAnimateManager.Add(rotateAnim);
    //雾
    g_valueAnimateManager.Add(fogPhase1);
    g_valueAnimateManager.Add(fogPhase2);
}

void Level_OnUpdate(){
    if(!g_said){
        g_said = true;
        g_cycleStartTime = GetGameTime();
        SetupDayNightAnimate();
        //PrintSystemText("昼夜系统已启动");
    }
    
    // 检查是否需要重启循环
    if (g_said) {
        float currentTime = GetGameTime();
        if (currentTime - g_cycleStartTime >= G_Time) {
            g_cycleStartTime = currentTime;
            SetupDayNightAnimate();
        }
    }

    g_valueAnimateManager.Update();
}

// 退出游戏时关闭昼夜系统
void Level_OnStop(){
    g_said = false;
    //清理动画，防止重进场景后闪烁
    g_valueAnimateManager.ClearAll();
}